function clearFrame() {  
  while (document.body.firstChild) {
    document.body.removeChild(document.body.firstChild);
  }
  window.scrollTo(0, 0);
}

function loadVerb(number, vname, dobj, prep, iobj) {
  clearFrame();
  var title = number.replace('obj', '#') + ":" + vname;
  setTitle(title);
  var result = getVerb(number, vname, dobj, prep, iobj);
  if (!result) {
    alert("Unable to locate verb.");
    return;
  }
  var perms = result[0];
  var owner = result[1];
  var code = result[2];
  owner = owner.replace('obj', '#');
  owner = parent.TreeFrame.treeGetOrCreateObject(owner, 'inheritance');
  owner = owner.firstChild.title;
  displayVerbInfo(dobj, prep, iobj, perms, owner);
  displayVerbCode(code);
}

function getVerb(number, vname, dobj, prep, iobj) {
  var xmlNode = parent.getXmlNodeById(number);
  if (!xmlNode) {
    return null;
  }
  var childXmlNodeLength = xmlNode.childNodes.length;
  var childXmlNode
  var code;
  nextVerb:
  for (var x = 0; x < childXmlNodeLength; x++) {
    childXmlNode = xmlNode.childNodes[x];
    if (childXmlNode.nodeName == "VERB" &&
        (!dobj || (childXmlNode.getAttribute('DOBJ') == dobj)) &&
        (!prep || (childXmlNode.getAttribute('PREP') == prep)) &&
        (!iobj || (childXmlNode.getAttribute('IOBJ') == iobj))) {
      // We have a verb on the correct object with matching args.
      // Take a look at the name.
      var grandchildXmlNodeLength = childXmlNode.childNodes.length;
      var grandchildXmlNode;
      var nameMatch = false;
      code = "";
      for (var y = 0; y < grandchildXmlNodeLength; y++) {
        grandchildXmlNode = childXmlNode.childNodes[y];
        if (grandchildXmlNode.nodeName == "NAME" && grandchildXmlNode.firstChild) {
          if (grandchildXmlNode.firstChild.nodeValue != vname) {
            continue nextVerb;
          }
          nameMatch = true;
        } else if (grandchildXmlNode.nodeName == "CODE") {
          code = "";
          for (var z = 0; z < grandchildXmlNode.childNodes.length; z++) {
            code += grandchildXmlNode.childNodes[z].nodeValue;
          }
        }
      }
      if (nameMatch) {
        // Found it!
        var perms = childXmlNode.getAttribute('PERMS');
        var owner = childXmlNode.getAttribute('OWNER');
        return [perms, owner, code];
      }
    }
  }
  return null;
}

function setTitle(title) {
  var node = document.getElementById('title');
  if (!node) {
    node = document.createElement("DIV");
    node.id = "title";
    document.body.appendChild(node);
  }
  if (typeof title != "object") {
    title = document.createTextNode(title);
  }
  node.appendChild(title);
}

function displayVerbInfo(dobj, prep, iobj, perms, owner) {
  var node;
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + dobj + " " + prep + " " + iobj));
  document.body.appendChild(node);
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + "Perms: " + perms));
  document.body.appendChild(node);
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + "Owner: " + owner));
  document.body.appendChild(node);
}

function displayVerbCode(code) {
  // Modified by KRJ to include textarea syntax highlighter by Christophe Dolivet;
  code = code.replace(/^\s+/, '').replace(/\s+$/, '');
  var node;
  if (code.length == 0) {
    node = document.createElement("P");
    node.className = "noVerbMsg";
    node.appendChild(document.createTextNode("Empty verb."));
    document.body.appendChild(node);
  } else {
    code = indentCode(code);
    //code = numberCode(code);
    node = document.createElement("textarea");
    node.id = "code";
	node.name="code";
	node.rows="20";
	node.cols="80";
    node.appendChild(document.createTextNode(code));
    // But IE doesn't understand linebreaks.  I hate IE.
    //node = document.createElement("DIV");
    //code = code.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
    //code = code.replace(/\n/g, "\r\n");
    //node.innerHTML = "<PRE id='code'>" + code + "</PRE>";
    document.body.appendChild(node);
    // initialize textarea for syntax highlighting;
	editAreaLoader.init({
	id : "code"		        // textarea id
	,syntax: "perl"			// syntax to be uses for highliting
	,start_highlight: true	// to display with highlight mode on start-up
    ,show_line_colors: true
    ,word_wrap: true
	,is_editable: true      // if false removes toolbar
    });
    //colourcode("code");
  }
}

function indentCode(code) {
  var lines = code.split("\n");
  var indent = "";
  for (var x = 0; x < lines.length; x++) {
    // Strip existing indent.
    lines[x] = lines[x].replace(/^\s+/, '');
    if (lines[x].match(/^(endif|else|elseif|endwhile|endfor|endfork|endtry|except)( |$)/)) {
      // Decrease indent on this line.
      indent = indent.substring(2);
    }
    lines[x] = indent + lines[x];
    if (lines[x].match(/^\s*(if|else|elseif|while|for|fork|try|except)( |$)/)) {
      // Increase indent on next line.
      indent += "  ";
    }
  }
  return lines.join("\n");
}

function numberCode(code) {
  var lines = code.split("\n");
  // Maximum number of lines is 2147483648 (10 digits)
  var spaces = "          ".substring(0, lines.length.toString().length);
  for (var x = 0; x < lines.length; x++) {
    lines[x] = (x+1) + ": " + spaces.substring((x+1).toString().length) + lines[x];
  }
  return lines.join("\n");
}


function loadProp(number, pname) {
  clearFrame();
  var title = number.replace('obj', '#') + "." + pname;
  setTitle(title);
  var result = getProp(number, pname);
  if (!result) {
    alert("Unable to locate property.");
    return;
  }
  var perms = result[0];
  var owner = result[1];
  var value = result[2];
  owner = owner.replace('obj', '#');
  owner = parent.TreeFrame.treeGetOrCreateObject(owner, 'inheritance');
  owner = owner.firstChild.title;
  var xmlNode = parent.getXmlNodeById(number);
  var clear = "No";
  while (!value && xmlNode) {
    xmlNode = xmlNode.parentNode;
    number = xmlNode.getAttribute('ID');
    if (!number) {
      // Database is internally inconsistent;
      break;
    }
    result = getProp(number, pname);
    if (!result) {
      // Database is internally inconsistent;
      break;
    }
    value = result[2];
    clear = "Yes";
  }
  displayPropInfo(perms, owner, clear);
  displayPropValue(value);
}

function getProp(number, pname) {
  var xmlNode = parent.getXmlNodeById(number);
  if (!xmlNode) {
    return null;
  }
  var childXmlNodeLength = xmlNode.childNodes.length;
  var childXmlNode
  var value;
  nextProp:
  for (var x = 0; x < childXmlNodeLength; x++) {
    childXmlNode = xmlNode.childNodes[x];
    if (childXmlNode.nodeName == "PROPERTY") {
      // We have a prop on the correct object.
      // Take a look at the name.
      var grandchildXmlNodeLength = childXmlNode.childNodes.length;
      var grandchildXmlNode;
      var nameMatch = false;
      value = "";
      for (var y = 0; y < grandchildXmlNodeLength; y++) {
        grandchildXmlNode = childXmlNode.childNodes[y];
        if (grandchildXmlNode.nodeName == "NAME" && grandchildXmlNode.firstChild) {
          if (grandchildXmlNode.firstChild.nodeValue != pname) {
            continue nextProp;
          }
          nameMatch = true;
        } else if (grandchildXmlNode.nodeName == "VALUE") {
          value = "";
          for (var z = 0; z < grandchildXmlNode.childNodes.length; z++) {
            value += grandchildXmlNode.childNodes[z].nodeValue;
          }
        }
      }
      if (nameMatch) {
        // Found it!
        var perms = childXmlNode.getAttribute('PERMS') || "";
        var owner = childXmlNode.getAttribute('OWNER');
        return [perms, owner, value];
      }
    }
  }
  return null;
}

function displayPropInfo(perms, owner, clear) {
  var node;
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + "Perms: " + perms));
  document.body.appendChild(node);
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + "Owner: " + owner));
  document.body.appendChild(node);
  node = document.createElement("DIV");
  node.appendChild(document.createTextNode("\xA0 " + "Clear: " + clear));
  document.body.appendChild(node);
}

function displayPropValue(value) {
// Modified by KRJ to include textarea syntax highlighter by Christophe Dolivet;
  value = value.replace(/^\s+/, '').replace(/\s+$/, '');
  // Two passes to replace spaces which would otherwise collapse with &nbsp;
  value = value.replace(/  /g, "\xA0 ");
  value = value.replace(/  /g, "\xA0 ");

  var node;
  //node = document.createElement("P");
  //node.id = "property";
  node = document.createElement("textarea");
    node.id = "property";
	node.name="property";
	node.rows="20";
	node.cols="80";
    node.appendChild(document.createTextNode(value));
  //node.appendChild(document.createTextNode(value));
  document.body.appendChild(node);
  editAreaLoader.init({
	id : "property"		        // textarea id
	,syntax: "perl"			    // syntax to be uses for highliting
	,start_highlight: true	    // to display with highlight mode on start-up
    ,show_line_colors: true
    ,word_wrap: true
	,is_editable: true          // if false removes toolbar
  });
  //colourcode("property");
}

