var regexp_whitespace = /^ +/;
var regexp_number = /^-?[0-9]+(.[0-9]+)?/;
var regexp_operator = /^(\.\.|<=|>=|==|!=|&&|\|\||[\[\]\(\)\{\}\!\^\%\*\.\=\+\:\;\<\>\/\?\`\'\|\,\@\$\-])/;
var regexp_word = /^[a-zA-Z_][a-zA-Z_0-9]*/;
var regexp_object = /^#-?[0-9]+/;
var regexp_string = /^"(\\.|[^"\\])*"/;
var regexp_linenumber = /^[0-9]+: */;

var moo_errors = " E_RANGE E_RECMOVE E_NONE E_PROPNF E_QUOTA E_DIV E_ARGS E_VARNF E_VERBNF E_PERM E_INVIND E_NACC E_TYPE E_FLOAT E_INVARG E_MAXREC ";
var moo_keywords = " for endfor if else elseif endif while endwhile fork endfork try except finally endtry break continue in ANY return ";
var moo_builtins = " disassemble log_cache_stats verb_cache_stats call_function raise suspend read seconds_left ticks_left pass set_task_perms caller_perms callers task_stack function_info load_server_options value_bytes value_hash string_hash binary_hash decode_binary encode_binary length setadd setremove listappend listinsert listdelete listset equal is_member tostr toliteral match rmatch substitute crypt index rindex strcmp strsub listiassoc listassoc server_log security_log toint tonum tofloat min max abs random time ftime ctime floatstr sqrt sin cos tan asin acos atan sinh cosh tanh exp log log10 ceil floor trunc toobj typeof create recycle object_bytes valid parent children chparent max_object players is_player set_player_flag move properties property_info set_property_info add_property delete_property clear_property is_clear_property server_version renumber reset_max_object memory_usage shutdown dump_database db_disk_size open_network_connection connected_players connected_seconds idle_seconds connection_name notify boot_player set_connection_option connection_option connection_options listen unlisten listeners buffered_output_length setuid task_id queued_tasks kill_task output_delimiters queue_info resume force_input flush_input verbs verb_info set_verb_info verb_args set_verb_args add_verb delete_verb verb_code set_verb_code eval new_waif file_version file_open file_close file_name file_openmode file_readline file_readlines file_writeline file_read file_write file_flush file_seek file_tell file_eof file_list file_mkdir file_rmdir file_remove file_rename file_chmod file_size file_mode file_type file_last_access file_last_modify file_last_change file_stat ";

function code2tokens(code) {
  // Break a chunk of plain-text Moo code into a list of tokens and their types.
  // e.g. 'tostr("I am", player);' -> ["tostr", "builtin", "(", "operator", "\"I am\"", "string", ",", "operator", " ", "whitespace", "player", "word", ")", "operator", ";", "operator"]
  var tokens = [];
  var m;
  if (m = regexp_linenumber.exec(code)) {
    tokens.push(m[0], "linenumber");
    code = code.substring(m[0].length);
  }
  // Comment_index: -2 = not possible; -1 = possible future, n = possible found 
  var comment_index = -1;
  while (code) {
    if (m = regexp_whitespace.exec(code)) {
      tokens.push(m[0], "whitespace");
      comment_index = -1;
    } else if (m = regexp_number.exec(code)) {
      tokens.push(m[0], m[0].indexOf(".") == -1 ? "integer" : "float");
      comment_index = -2;
    } else if (m = regexp_operator.exec(code)) {
      tokens.push(m[0], "operator");
      if (m[0] != ";") comment_index = -2;
    } else if (m = regexp_word.exec(code)) {
      if (tokens.length && tokens[tokens.length-2] == "$") {
        tokens[tokens.length-2] += m[0];
        tokens[tokens.length-1] = "system";
        code = "$"+code;
      } else {
        tokens.push(m[0]);
        if (moo_keywords.indexOf(" "+m[0]+" ") != -1)
          tokens.push("keyword");
        else if (moo_errors.indexOf(" "+m[0]+" ") != -1)
          tokens.push("error");
        else if (moo_builtins.indexOf(" "+m[0]+" ") != -1 && (tokens.length == 1 || tokens[tokens.length-3] != ":") && (code.substring(m[0].length, m[0].length+1) == "("))
          tokens.push("builtin");
        else
          tokens.push("word");
      }
      comment_index = -2;
    } else if (m = regexp_object.exec(code)) {
      tokens.push(m[0], "object");
      comment_index = -2;
    } else if (m = regexp_string.exec(code)) {
      tokens.push(m[0], "string");
      comment_index = (comment_index == -1) ? tokens.length-1 : -2;
    } else {
      // Syntax error, take everything to the end of the line.
      tokens.push(code, "syntax");
    }
    code = code.substring(tokens[tokens.length-2].length);
  }
  if (comment_index > 0)
    tokens[comment_index] = "comment";
  return tokens;
}

function tokens2html(tokens) {
  // Takes a list of tokens and types, and returns pretty HTML.
  var html = "";
  var token, type;
  var brackets = [];
  for (var x=0; x<tokens.length; x+=2) {
    token = tokens[x];
    type = tokens[x+1];
    if (type == "builtin") {
      // Merge the builtin function's bracket
      token += "(";
      x += 2;
      brackets.push("builtin");
    } else if (type == "keyword" && tokens.length > x + 4 && tokens[x+2] == " " && (tokens[x+4] == "(" || tokens[x+4] == "[")) {
      // Merge the if/while/for/etc with its bracket
      token += " " + tokens[x+4];
      x += 4;
      brackets.push("keyword");
    }
    if (token == "(" || token == "[")
      brackets.push("operator");
    if (token == ")" || token == "]")
      type = brackets.pop() || "syntax";

    if (type == "whitespace" || type=="linebreak") {
      html += token;
    } else {
      token = token.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;");
      html += "<SPAN CLASS='code_"+type+"'>"+token+"</SPAN>";
    }
  }
  return html;
}

function colourcode(id) {
  var el = document.getElementById(id);
  if (el != null) {
    var code = el.innerHTML;
    code = code.replace(/&quot;/g,'"').replace(/&lt;/g,"<").replace(/&gt;/g,">").replace(/&amp;/g,"&");
    var lines = code.split(/\r?\n/);
    var tokens = [];
    for (var x=0; x<lines.length; x++) {
      tokens = tokens.concat(code2tokens(lines[x]), "\n", "linebreak");
    }
    code = tokens2html(tokens);
    // An IE bug forces us to encode whitespaces or else they will vanish.  I hate IE.
    code = code.replace(/\n/g, "<br />");
    // Two passes to replace spaces which would otherwise collapse with &nbsp;
    code = code.replace(/  /g, "\xA0 ").replace(/  /g, "\xA0 ");
    el.innerHTML = code;
  }
}

